// localization implementation details, XPG7 version -*- C++ -*-

// Copyright (C) 2001-2021 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or (at your option)
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Under Section 7 of GPL version 3, you are granted additional
// permissions described in the GCC Runtime Library Exception, version
// 3.1, as published by the Free Software Foundation.

// You should have received a copy of the GNU General Public License and
// a copy of the GCC Runtime Library Exception along with this program;
// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
// <http://www.gnu.org/licenses/>.

/** @file bits/c++locale.h
 *  This is an internal header file, included by other library headers.
 *  Do not attempt to use it directly. @headername{locale}
 */

//
// ISO C++ 14882: 22.8  Standard locale categories.
//

// Written by Benjamin Kosnik <bkoz@redhat.com>
// Modified for DragonFly by John Marino <gnugcc@marino.st>
// Modified for XPG7 by Clément Chigot <clement.chigot@atos.net>

#ifndef _GLIBCXX_CXX_LOCALE_H
#define _GLIBCXX_CXX_LOCALE_H 1

#pragma GCC system_header

#include <clocale>
#include <cstdlib>
#ifdef _GLIBCXX_HAVE_XLOCALE_H
#include <xlocale.h>
#endif

#define _GLIBCXX_C_LOCALE_XPG7 1

// Assume that if strtof_l is missing then strtod_l and strold_l
// are missing too.
#ifndef _GLIBCXX_HAVE_STRTOF_L
extern "C" inline
float strtof_l (const char *__nptr, char **__endptr,
		locale_t locale)
{
  locale_t __old = uselocale(locale);
  float __res = strtof(__nptr, __endptr);
  uselocale(__old);
  return __res;
}

extern "C" inline
double strtod_l (const char *__nptr, char **__endptr,
		 locale_t locale)
{
  locale_t __old = uselocale(locale);
  double __res = strtod(__nptr, __endptr);
  uselocale(__old);
  return __res;
}

extern "C" inline
long double strtold_l (const char *__nptr, char **__endptr,
		       locale_t locale)
{
  locale_t __old = uselocale(locale);
  long double __res = strtold(__nptr, __endptr);
  uselocale(__old);
  return __res;
}
#endif

#define _GLIBCXX_NUM_CATEGORIES 0

namespace std _GLIBCXX_VISIBILITY(default)
{
_GLIBCXX_BEGIN_NAMESPACE_VERSION

#if defined(__DragonFly__) || defined(__FreeBSD__)
  typedef int*			__c_locale;
#else
  typedef locale_t			__c_locale;
#endif

  // Convert numeric value of type double and long double to string and
  // return length of string.  If vsnprintf is available use it, otherwise
  // fall back to the unsafe vsprintf which, in general, can be dangerous
  // and should be avoided.
  inline int
  __convert_from_v(const __c_locale& __cloc, char* __out,
		   const int __size __attribute__ ((__unused__)),
		   const char* __fmt, ...)
  {
    __c_locale __old = (__c_locale)uselocale((locale_t)__cloc);

    __builtin_va_list __args;
    __builtin_va_start(__args, __fmt);

#if _GLIBCXX_USE_C99_STDIO
    const int __ret = __builtin_vsnprintf(__out, __size, __fmt, __args);
#else
    const int __ret = __builtin_vsprintf(__out, __fmt, __args);
#endif

    __builtin_va_end(__args);

    uselocale((locale_t)__old);
    return __ret;
  }

_GLIBCXX_END_NAMESPACE_VERSION
} // namespace

#endif
